<?php

/**
 * @file
 * Handle the base functionality for querying the Pin API.
 */


/**
 * A simple static class to wrap our endpoints.
 */
class CommercePinEndpoints {
  const endpoint_live = 'api.pin.net.au/1';
  const endpoint_test = 'test-api.pin.net.au/1';
}

/**
 * The base class for a Pin transaction.
 */
abstract class CommercePinTransactionBase {

  private $secret_key = FALSE;

  /**
   * To create a new transaction we require the API details and a flag to
   * specify if we are in test mode or not.
   */
  function __construct($secret_key, $test_mode = FALSE) {
    $this->secret_key = $secret_key;
    $this->test_mode = $test_mode;
  }

  /**
   * Get the endpoint we will be using for the transaction including the
   * authentication details.
   */
  function getApiBase() {

    $url = $this->test_mode
      ? CommercePinEndpoints::endpoint_test
      : CommercePinEndpoints::endpoint_live;

    return 'https://' . $this->secret_key . ':@' . $url;
  }

  /**
   * Query the API.
   */
  function queryPinApi($endpoint, $variables) {

    // The endpoint URL we will query.
    $endpoint_url = $this->getApiBase() . '/' . $endpoint . '/';

    // Add some standard variables to our request variables.
    $request_variables = array(
      'ip_address' => $_SERVER['REMOTE_ADDR'],
    ) + $variables;

    $encoded_request_variables = json_encode($request_variables);

    // Get a curl request and set the correct endpoint.
    $request = $this->createCurlPostRequest();

    curl_setopt($request, CURLOPT_URL, $endpoint_url);
    curl_setopt($request, CURLOPT_POSTFIELDS, $encoded_request_variables);
    curl_setopt($request, CURLOPT_HTTPHEADER,
      array(
        'Content-Length: ' . strlen($encoded_request_variables),
        'Content-Type: application/json',
      )
    );

    // Get the response from Pin.
    $response = json_decode(curl_exec($request));

    return $response;
  }


  /**
   * Create a curl POST request and return a curl handler.
   */
  function createCurlPostRequest() {
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    return $ch;
  }

}
